import math
import struct
import numpy as np

def binary(number: float):
    """Print the mantissa and exponent of a float.

    Parameters
    ----------
    number : float
        A floating point number

    Returns
    -------
    None : NoneType
        Returns nothing, prints output.

    Examples
    --------
    >>> binary(1.25)
     Decimal: 1.25 x 2^0
      Binary: 1.01 x 2^0

        Sign: 0 (+)
    Mantissa: 01 (0.25)
    Exponent: 0 (0)
    """

    # 1. The struct module packs a float into a string of bytes.
    packed = struct.pack("!d", float(number))
    # 2. The bytes are converted to integers.
    integers = [c for c in packed]
    # 3. The integers are converted to binary strings.
    binaries = [bin(i) for i in integers]
    # 4. The "0b" prefix is stripped from the binary strings.
    stripped_binaries = [s.replace("0b", "") for s in binaries]
    # 5. The binary strings are padded with leading zeros to ensure they are 8 characters long.
    padded = [s.rjust(8, "0") for s in stripped_binaries]
    # 6. The padded binary strings are concatenated into one giant string.
    final = "".join(padded)
    assert len(final) == 64, "something went wrong..."
    # 7. The giant string is split into 3 parts: the sign, the exponent, and the mantissa.
    sign, exponent_plus_1023, mantissa = final[0], final[1:12], final[12:]
    sign_str = "" if int(sign) == 0 else "-"
    mantissa_base10 = (
        int(mantissa, 2) / 2 ** 52
    )  
    mantissa_base10_str = str(mantissa_base10)[2:] 
    # 8. The sign is converted from binary to decimal.
    mantissa_index = mantissa.rfind("1")
    # 9. The mantissa is converted from binary to decimal.
    mantissa_index = 0 if mantissa_index == -1 else mantissa_index
    # 10. The exponent is converted from binary to decimal.
    exponent_base10 = int(exponent_plus_1023, 2) - 1023
    print(f"\n - Decimal: {sign_str}1.{mantissa_base10_str} x 2^{exponent_base10}")
    print(
        f" - Binary: {sign_str}1.{mantissa[:mantissa_index + 1]} x 2^{exponent_base10:b}"
    )
    print(f" - Sign: {sign} ({'+' if sign == '0' else '-'})")
    print(f" - Mantissa: {mantissa[:mantissa_index + 1]} ({mantissa_base10})")
    print(f" - Exponent: {exponent_base10:b} ({exponent_base10})\n")


def calc_spacing(number: float):
    """ Calculate the spacing between two floating point numbers.
    """
    return np.nextafter(number, 2 * number) - number



# -*- coding: utf-8 -*-
from setuptools import setup

package_dir = \
{'': 'src'}

packages = \
['mltz_base']

package_data = \
{'': ['*']}

install_requires = \
['numpy>=1.24.0', 'pandas>=1.4.0']

setup_kwargs = {
    'name': 'mltz-base',
    'version': '0.1.24',
    'description': 'Base Package for ML related dev',
    'long_description': '\n[![codecov](https://codecov.io/github/tzoght/mltz-base/branch/main/graph/badge.svg?token=UB03POGOUB)](https://codecov.io/github/tzoght/mltz-base)\n[![ci-cd](https://github.com/tzoght/mltz-base/actions/workflows/ci-cd.yml/badge.svg?branch=main)](https://github.com/tzoght/mltz-base/actions/workflows/ci-cd.yml) [![Documentation Status](https://readthedocs.org/projects/mltz-base/badge/?version=latest)](https://mltz-base.readthedocs.io/en/latest/?badge=latest) ![PyPI](https://img.shields.io/pypi/v/mltz_base)\n\n# mltz_base\n\nBase Package for ML related dev\n\n## Installation\n\n```bash\n$ pip install mltz_base\n```\nfrom  [Pypi repo](https://pypi.org/manage/project/mltz-base/releases/)\n\n## Usage\n\n- Coming soon\n\n## Contributing\n\nInterested in contributing? Check out the contributing guidelines. Please note that this project is released with a Code of Conduct. By contributing to this project, you agree to abide by its terms.\n\n## License\n\n`mltz_base` was created by Tony Zoght. It is licensed under the terms of the MIT license.\n\n## Credits\n\n`mltz_base` was created with [`cookiecutter`](https://cookiecutter.readthedocs.io/en/latest/) and the `py-pkgs-cookiecutter` [template](https://github.com/py-pkgs/py-pkgs-cookiecutter).\n\n\n## Code Coverage\n* [CodeCov](https://app.codecov.io/github/tzoght/mltz-base) \n',
    'author': 'Tony Zoght',
    'author_email': 'None',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'None',
    'package_dir': package_dir,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.9',
}


setup(**setup_kwargs)
